package com.persagy.utils.condition;

import com.alibaba.fastjson.JSONObject;
import com.persagy.entity.Condition;
import com.persagy.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @description: 报警定义util类
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2021/1/8 上午11:21
 * @version: V1.0
 **/
public class ConditionUtils {

    /**
     * 生效类型：always
     */
    public static final String EFFECT_TYPE_ALWAYS = "always";

    /**
     * 生效类型：period
     */
    public static final String EFFECT_TYPE_PERIOD = "period";

    /**
     * @description: 获取有效期类型
     * @param: condition
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午3:38
     * @version: V1.0
     */
    public static String getEffectType(Condition condition) {
        if (condition == null) {
            return null;
        }
        JSONObject effectTime = condition.getEffectTime();
        if (effectTime == null) {
            return null;
        }

        return effectTime.getString("type");
    }

    /**
     * @description: 获取有效期开始时间
     * @param: condition
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午3:44
     * @version: V1.0
     */
    public static String getEffectStartTime(Condition condition) {
        if (condition == null) {
            return null;
        }
        JSONObject effectTime = condition.getEffectTime();
        if (effectTime == null) {
            return null;
        }

        String type = effectTime.getString("type");
        if (EFFECT_TYPE_PERIOD.equals(type)) {
            JSONObject period = effectTime.getJSONObject("period");
            if (period == null) {
                return null;
            }
            return period.getString("startTime");
        }

        return null;
    }

    /**
     * @description: 获取有效期结束时间
     * @param: condition
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午3:44
     * @version: V1.0
     */
    public static String getEffectEndTime(Condition condition) {
        if (condition == null) {
            return null;
        }
        JSONObject effectTime = condition.getEffectTime();
        if (effectTime == null) {
            return null;
        }

        String type = effectTime.getString("type");
        if (EFFECT_TYPE_PERIOD.equals(type)) {
            JSONObject period = effectTime.getJSONObject("period");
            if (period == null) {
                return null;
            }
            return period.getString("endTime");
        }

        return null;
    }

    /**
     * @description: 判断报警条件中的过期设置是否生效
     * @param: condition
     * @return: boolean
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午2:32
     * @version: V1.0
     */
    public static boolean expireSetEffective(Condition condition) {
        if (condition == null) {
            return false;
        }

        JSONObject effectTime = condition.getEffectTime();
        if (effectTime == null) {
            return false;
        }

        String type = effectTime.getString("type");
        if (StringUtils.isBlank(type)) {
            return false;
        }
        // 只有type=period，且设置的生效时间在"000000" -> "235959"之间，过期设置才有效
        if (EFFECT_TYPE_PERIOD.equals(type)) {
            String startTime = getEffectStartTime(condition);
            String endTime = getEffectEndTime(condition);
            // 时间不合法，认为过期设置不生效
            if (!validateTime(startTime) || !validateTime(endTime)) {
                return false;
            }

            // 一天的开始和结束时间
            String dayStartTime = "000000";
            String dayEndTime = "235959";
            // 设置的生效时间为"000000" -> "235959"，过期设置不生效
            if (dayStartTime.equals(startTime) && dayEndTime.equals(endTime)) {
                return false;
            }

            // 前面的校验都通过，并且开始时间<结束时间，过期设置才有效
            return startTime.compareTo(endTime) < 0;
        }

        return false;
    }

    /**
     * @description: 校验时间是否合法（是否为6位数字）
     * @param: time
     * @return: boolean
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午3:13
     * @version: V1.0
     */
    private static boolean validateTime(String time) {
        if (StringUtils.isBlank(time)) {
            return false;
        }
        String regex = "^\\d{6}$";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(time);
        return matcher.matches();
    }

    /**
     * @description: 判断时间是否在有效期内
     * @param: condition
     * @param: datetime
     * @return: boolean
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午3:31
     * @version: V1.0
     */
    public static boolean inEffectiveTime(Condition condition, String datetime) {
        if (StringUtils.isBlank(datetime)) {
            return false;
        }
        String time = DateUtils.getSdfTimeNotDate(datetime);
        // 如果时间格式有问题，认为不在有效期内
        if (!validateTime(time)) {
            return false;
        }

        String effectType = getEffectType(condition);
        // 如果有效期类型为always, 一定在有效期内
        if (EFFECT_TYPE_ALWAYS.equals(effectType)) {
            return true;
        }

        if (EFFECT_TYPE_PERIOD.equals(effectType)) {
            String effectStartTime = getEffectStartTime(condition);
            String effectEndTime = getEffectEndTime(condition);
            // 如果报警条件中的有效期开始时间或者结束时间格式有问题，认为不在有效期
            if (!validateTime(effectStartTime) || !validateTime(effectEndTime)) {
                return false;
            }
            return effectStartTime.compareTo(time) <= 0 && effectEndTime.compareTo(time) >= 0;
        }

        return false;
    }

    public static void main(String[] args) {
        Condition condition = new Condition();
        JSONObject effectTime = new JSONObject();
//        JSONObject period = new JSONObject();
//        period.put("startTime", "000000");
//        period.put("endTime", "115958");
////        effectTime.put("type", EFFECT_TYPE_PERIOD);
//        effectTime.put("type", EFFECT_TYPE_ALWAYS);
//        effectTime.put(EFFECT_TYPE_PERIOD, period);
        condition.setEffectTime(effectTime);
        boolean b = inEffectiveTime(condition, "20210108115959");
        System.out.println(b);
    }
}
